"""
This file contains a number of tweaks in order to tune orekit with jpype according to preferences.

It is assumed that the JVM is running.
Importing this file will automatically implement the changes.

"""
import numpy as np
import datetime, math
import orekit_jpype


from jpype._jcustomizer import JImplementationFor, JConversion
import jpype, _jpype

from jpype import JArray, JDouble

from org.orekit.time import TimeScalesFactory, AbsoluteDate
from orekit_jpype.pyhelpers import absolutedate_to_datetime, datetime_to_absolutedate

import os
dirpath = os.path.dirname(os.path.abspath(__file__))


# Change representation of object (__repr__) to include toString()
def _JObjectRepr(obj) -> str:
    # Function to generate the __repr__ string for objects in interactive mode

    return f"<{obj.getClass().getSimpleName()}: {obj.toString()}>"


# Monkey patch the base class
_jpype._JObject.__repr__ = _JObjectRepr

# Create a top level function JArray_double to mimic JCC backend
orekit_jpype.JArray_double = JArray(JDouble)


# Some helper methods on selected classes
@JImplementationFor("org.orekit.time.AbsoluteDate")
class _JAbsoluteDate(object):

    def to_datetime(self) -> datetime.datetime:
        """

        Returns: The AbsoluteDate as a Python datetime

        """
        # offset = self.durationFrom(AbsoluteDate.JAVA_EPOCH)
        utc = TimeScalesFactory.getUTC()
        or_comp = self.getComponents(utc)
        or_date = or_comp.getDate()
        or_time = or_comp.getTime()
        seconds = or_time.getSecond()
        return datetime.datetime(or_date.getYear(),
                                 or_date.getMonth(),
                                 or_date.getDay(),
                                 or_time.getHour(),
                                 or_time.getMinute(),
                                 int(math.floor(seconds)),
                                 int(1000000.0 * (seconds - math.floor(seconds))))


@JImplementationFor("double[][]")
class _JDouble2DArray(object):

    def to_numpy(self) -> np.ndarray:
        """
        Get the Java Double 2D Array as a Python numpy array

        Returns: the Double Array as numpy 2D array

        """
        return np.array(self)

    def __repr__(self):
        return f"<{self.getClass().getSimpleName()}: {np.array(self).__str__()}>"

# @JImplementationFor("double[]")
# class _JDoubleArray(object):
#
#     def to_numpy(self):
#         """
#         Get the Java Double Array as a Python numpy array
#
#         Returns: the Double Array as numpy array
#
#         """
#         return np.array(self)
#
#    def __repr__(self):
#        return f"<{self.getClass().getSimpleName()}: {np.array(self).__str__()}>"


# Conversions
@JConversion("org.orekit.time.AbsoluteDate", instanceof=datetime.datetime)
def _JADConversion(jcls, obj):
    return datetime_to_absolutedate(obj)  # AbsoluteDate.JAVA_EPOCH.shiftedBy(obj.replace(tzinfo=datetime.timezone.utc).timestamp())
